#!/usr/bin/env node

import { ArgumentParser } from 'argparse';
import chokidar from 'chokidar';
import express, { Request, Response } from 'express';
import marked from 'marked';
import WebSocket from 'ws';
import hljs from 'highlight.js';

import * as fs from 'fs';
import * as path from 'path';

import template from './template';

const parser = new ArgumentParser({
  add_help: true,
  description: 'Realtime Markdown preview',
});

parser.add_argument('file');
parser.add_argument('-p', '--port', {
  default: 1729,
  help: 'The port to run on',
  type: 'int',
});
parser.add_argument('-t', '--theme', {
  default: 'zenburn',
  help:
    'The theme to use for highlighted code blocks. A full list of available themes can be found at https://highlightjs.org/static/demo.',
  type: 'str',
});

const { file, port, theme } = parser.parse_args();
const filepath = path.resolve(file);

const getRenderedHTML = () => {
  const fileContents = fs.readFileSync(filepath, 'utf-8');
  return marked(fileContents, {
    highlight: (code, language) => {
      const validLanguage = hljs.getLanguage(language) ? language : 'plaintext';
      return hljs.highlight(validLanguage, code).value;
    },
  });
};

const wss = new WebSocket.Server({
  port: 40510,
});

wss.on('connection', (ws) => {
  chokidar.watch(filepath).on('change', () => {
    ws.send(getRenderedHTML(), (err) => {
      if (err) console.log(err);
    });
  });
});

const app = express();

app.get('/', (_req: Request, res: Response) => {
  res.send(template(path.basename(file), getRenderedHTML(), theme));
});

app.listen(port, () => {
  console.log(`Listening at http://localhost:${port}`);
});
